#!/usr/bin/env python
#-*- coding: utf-8 -*-

# Stéganô - Stéganô is a basic Python Steganography module.
# Copyright (C) 2010-2013  Cédric Bonhomme - http://cedricbonhomme.org/
#
# For more information : http://bitbucket.org/cedricbonhomme/stegano/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.1 $"
__date__ = "$Date: 2010/03/24 $"
__license__ = "GPLv3"

# Thanks to: http://www.julesberman.info/spec2img.htm

def hide(img, img_enc, copyright="http://bitbucket.org/cedricbonhomme/stegano", \
            secret_message = None, secret_file = None):
    """
    """
    import shutil
    import datetime
    from zlib import compress
    from zlib import decompress
    from base64 import b64encode
    from .exif.minimal_exif_writer import MinimalExifWriter

    if secret_file != None:
        with open(secret_file, "r") as f:
            secret_file_content = f.read()
    text = "\nImage annotation date: "
    text = text + str(datetime.date.today())
    text = text  + "\nImage description:\n"
    if secret_file != None:
        text = compress(b64encode(text + secret_file_content))
    else:
        text = compress(b64encode(text + secret_message))

    try:
        shutil.copy(img, img_enc)
    except Exception as e:
        print(("Impossible to copy image:", e))
        return

    f = MinimalExifWriter(img_enc)
    f.removeExif()
    f.newImageDescription(text)
    f.newCopyright(copyright, addYear = 1)
    f.process()


def reveal(img):
    """
    """
    from base64 import b64decode
    from zlib import decompress
    from .exif.minimal_exif_reader import MinimalExifReader
    try:
        g = MinimalExifReader(img)
    except:
        print("Impossible to read description.")
        return
    print((b64decode(decompress(g.imageDescription()))))
    print(("\nCopyright " + g.copyright()))
    #print g.dateTimeOriginal()s


if __name__ == "__main__":
    # Point of entry in execution mode.
    from optparse import OptionParser
    parser = OptionParser(version=__version__)
    parser.add_option('--hide', action='store_true', default=False,
                      help="Hides a message in an image.")
    parser.add_option('--reveal', action='store_true', default=False,
                      help="Reveals the message hided in an image.")
    # Original image
    parser.add_option("-i", "--input", dest="input_image_file",
                    help="Input image file.")
    # Image containing the secret
    parser.add_option("-o", "--output", dest="output_image_file",
                    help="Output image containing the secret.")

    # Secret raw message to hide
    parser.add_option("-m", "--secret-message", dest="secret_message",
                    help="Your raw secret message to hide.")

    # Secret text file to hide.
    parser.add_option("-f", "--secret-file", dest="secret_file",
                    help="Your secret textt file to hide.")

    parser.set_defaults(input_image_file = './pictures/Elisha-Cuthbert.jpg',
                        output_image_file = './pictures/Elisha-Cuthbert_enc.jpg',
                        secret_message = '', secret_file = '')

    (options, args) = parser.parse_args()

    if options.hide:
        if options.secret_message != "" and options.secret_file == "":
            hide(img=options.input_image_file, img_enc=options.output_image_file, \
                    secret_message=options.secret_message)
        elif options.secret_message == "" and options.secret_file != "":
            hide(img=options.input_image_file, img_enc=options.output_image_file, \
                    secret_file=options.secret_file)

    elif options.reveal:
        reveal(img=options.input_image_file)


#!/usr/bin/env python
#-*- coding: utf-8 -*-

# Stéganô - Stéganô is a basic Python Steganography module.
# Copyright (C) 2010-2013  Cédric Bonhomme - http://cedricbonhomme.org/
#
# For more information : http://bitbucket.org/cedricbonhomme/stegano/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.4 $"
__date__ = "$Date: 2011/12/28 $"
__license__ = "GPLv3"

import sys

from PIL import Image

import tools
import generators

def hide(input_image_file, message, generator_function):
    """
    Hide a message (string) in an image with the
    LSB (Least Significant Bit) technique.
    """
    img = Image.open(input_image_file)
    img_list = list(img.getdata())
    width, height = img.size
    index = 0

    message = str(len(message)) + ":" + message
    #message_bits = tools.a2bits(message)
    message_bits = "".join(tools.a2bits_list(message))

    npixels = width * height
    if len(message_bits) > npixels * 3:
        raise Exception("""The message you want to hide is too long (%s > %s).""" % (len(message_bits), npixels * 3))

    generator = getattr(generators, generator_function)()
    
    while index + 3 <= len(message_bits) :
        generated_number = next(generator)
        (r, g, b) = img_list[generated_number]

        # Change the Least Significant Bit of each colour component.
        r = tools.setlsb(r, message_bits[index])
        g = tools.setlsb(g, message_bits[index+1])
        b = tools.setlsb(b, message_bits[index+2])

        # Save the new pixel
        img_list[generated_number] = (r, g , b)
        
        index += 3

    # create empty new image of appropriate format
    encoded = Image.new('RGB', (img.size))
        
    # insert saved data into the image
    encoded.putdata(img_list)
    
    return encoded
    


def reveal(input_image_file, generator_function):
    """
    Find a message in an image
    (with the LSB technique).
    """
    img = Image.open(input_image_file)
    img_list = list(img.getdata())
    width, height = img.size
    buff, count = 0, 0
    bitab = []
    limit = None

    generator = getattr(generators, generator_function)()

    while True:
        generated_number = next(generator)
        # color = [r, g, b]
        for color in img_list[generated_number]:
            buff += (color&1)<<(7-count)
            count += 1
            if count == 8:
                bitab.append(chr(buff))
                buff, count = 0, 0
                if bitab[-1] == ":" and limit == None:
                    try:
                        limit = int("".join(bitab[:-1]))
                    except:
                        pass
        if len(bitab)-len(str(limit))-1 == limit :
            return "".join(bitab)[len(str(limit))+1:]

    return ""

def write(image, output_image_file):
    """
    """
    try:
        image.save(output_image_file)
    except Exception as e:
        # If hide() returns an error (Too long message).
        print(e)

if __name__ == '__main__':
    # Point of entry in execution mode.
    from optparse import OptionParser
    parser = OptionParser(version=__version__)
    parser.add_option('--hide', action='store_true', default=False,
                      help="Hides a message in an image.")
    parser.add_option('--reveal', action='store_true', default=False,
                      help="Reveals the message hided in an image.")
    # Original image
    parser.add_option("-i", "--input", dest="input_image_file",
                    help="Input image file.")

    # Generator
    parser.add_option("-g", "--generator", dest="generator_function",
                    help="Generator")
                    
    # Image containing the secret
    parser.add_option("-o", "--output", dest="output_image_file",
                    help="Output image containing the secret.")

    # Non binary secret message to hide
    parser.add_option("-m", "--secret-message", dest="secret_message",
                    help="Your secret message to hide (non binary).")

    # Binary secret to hide (OGG, executable, etc.)
    parser.add_option("-f", "--secret-file", dest="secret_file",
                    help="Your secret to hide (Text or any binary file).")
    # Output for the binary binary secret.
    parser.add_option("-b", "--binary", dest="secret_binary",
                    help="Output for the binary secret (Text or any binary file).")

    parser.set_defaults(input_image_file = './pictures/Lenna.png',
                        generator_function = 'fermat',
                        output_image_file = './pictures/Lenna_enc.png',
                        secret_message = '', secret_file = '', secret_binary = "")

    (options, args) = parser.parse_args()


    if options.hide:
        if options.secret_message != "" and options.secret_file == "":
            secret = options.secret_message
        elif options.secret_message == "" and options.secret_file != "":
            secret = tools.binary2base64(options.secret_file)

        img_encoded = hide(options.input_image_file, secret, options.generator_function)
        try:
            img_encoded.save(options.output_image_file)
        except Exception as e:
            # If hide() returns an error (Too long message).
            print(e)

    elif options.reveal:
        try:
            secret = reveal(options.input_image_file, options.generator_function)
        except IndexError:
            print("Impossible to detect message.")
            exit(0)
        if options.secret_binary != "":
            data = tools.base642binary(secret)
            with open(options.secret_binary, "w") as f:
                f.write(data)
        else:
            print(secret)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

# Stéganô - Stéganô is a basic Python Steganography module.
# Copyright (C) 2010-2011  Cédric Bonhomme - http://cedricbonhomme.org/
#
# For more information : http://bitbucket.org/cedricbonhomme/stegano/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.2 $"
__date__ = "$Date: 2010/03/24 $"
__license__ = "GPLv3"

import sys

from PIL import Image

import tools

def hide(input_image_file, message):
    """
    Hide a message (string) in an image with the
    LSB (Least Significant Bit) technique.
    """
    img = Image.open(input_image_file)
    encoded = img.copy()
    width, height = img.size
    index = 0

    message = str(len(message)) + ":" + message
    #message_bits = tools.a2bits(message)
    message_bits = "".join(tools.a2bits_list(message))

    npixels = width * height
    if len(message_bits) > npixels * 3:
        raise Exception("""The message you want to hide is too long (%s > %s).""" % (len(message_bits), npixels * 3))

    for row in range(height):
        for col in range(width):

            if index + 3 <= len(message_bits) :

                # Get the colour component.
                (r, g, b) = img.getpixel((col, row))

                # Change the Least Significant Bit of each colour component.
                r = tools.setlsb(r, message_bits[index])
                g = tools.setlsb(g, message_bits[index+1])
                b = tools.setlsb(b, message_bits[index+2])

                # Save the new pixel
                encoded.putpixel((col, row), (r, g , b))

            index += 3

    return encoded

def reveal(input_image_file):
    """
    Find a message in an image
    (with the LSB technique).
    """
    img = Image.open(input_image_file)
    width, height = img.size
    buff, count = 0, 0
    bitab = []
    limit = None
    for row in range(height):
        for col in range(width):

            # color = [r, g, b]
            for color in img.getpixel((col, row)):
                buff += (color&1)<<(7-count)
                count += 1
                if count == 8:
                    bitab.append(chr(buff))
                    buff, count = 0, 0
                    if bitab[-1] == ":" and limit == None:
                        try:
                            limit = int("".join(bitab[:-1]))
                        except:
                            pass

            if len(bitab)-len(str(limit))-1 == limit :
                return "".join(bitab)[len(str(limit))+1:]
    return ""

def write(image, output_image_file):
    """
    """
    try:
        image.save(output_image_file)
    except Exception as e:
        # If hide() returns an error (Too long message).
        print(e)

if __name__ == '__main__':
    # Point of entry in execution mode.
    from optparse import OptionParser
    parser = OptionParser(version=__version__)
    parser.add_option('--hide', action='store_true', default=False,
                      help="Hides a message in an image.")
    parser.add_option('--reveal', action='store_true', default=False,
                      help="Reveals the message hided in an image.")
    # Original image
    parser.add_option("-i", "--input", dest="input_image_file",
                    help="Input image file.")
    # Image containing the secret
    parser.add_option("-o", "--output", dest="output_image_file",
                    help="Output image containing the secret.")

    # Non binary secret message to hide
    parser.add_option("-m", "--secret-message", dest="secret_message",
                    help="Your secret message to hide (non binary).")

    # Binary secret to hide (OGG, executable, etc.)
    parser.add_option("-f", "--secret-file", dest="secret_file",
                    help="Your secret to hide (Text or any binary file).")
    # Output for the binary binary secret.
    parser.add_option("-b", "--binary", dest="secret_binary",
                    help="Output for the binary secret (Text or any binary file).")

    parser.set_defaults(input_image_file = './pictures/Lenna.png',
                        output_image_file = './pictures/Lenna_enc.png',
                        secret_message = '', secret_file = '', secret_binary = "")

    (options, args) = parser.parse_args()


    if options.hide:
        if options.secret_message != "" and options.secret_file == "":
            secret = options.secret_message
        elif options.secret_message == "" and options.secret_file != "":
            secret = tools.binary2base64(options.secret_file)

        img_encoded = hide(options.input_image_file, secret)
        try:
            img_encoded.save(options.output_image_file)
        except Exception as e:
            # If hide() returns an error (Too long message).
            print(e)

    elif options.reveal:
        secret = reveal(options.input_image_file)
        if options.secret_binary != "":
            data = tools.base642binary(secret)
            with open(options.secret_binary, "w") as f:
                f.write(data)
        else:
            print(secret)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Stéganô - Stéganô is a basic Python Steganography module.
# Copyright (C) 2010-2013  Cédric Bonhomme - http://cedricbonhomme.org/
#
# For more information : http://bitbucket.org/cedricbonhomme/stegano/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.2 $"
__date__ = "$Date: 2011/12/28 $"
__revision__ = "$Date: 2012/12/14 $"
__license__ = "GPLv3"

import math
import itertools

def identity():
    """
    f(x) = x
    """
    n = 0
    while True:
        yield n
        n += 1

def Dead_Man_Walking():
    n = 0
    while True:
        yield n + 7
        n += 2
        
def OEIS_A000217():
    """
    http://oeis.org/A000217
    Triangular numbers: a(n) = C(n+1,2) = n(n+1)/2 = 0+1+2+...+n.
    """
    n = 0
    while True:
        yield (n*(n+1))//2
        n += 1

def fermat():
    """
    Generate the n-th Fermat Number.
    """
    y = 5
    while True:
        yield y
        y = pow(y-1,2)+1

def mersenne():
    """
    Generate 2^n-1.
    """
    y = 1
    while True:
        yield y
        y = 2*y + 1

def eratosthenes():
    """
    Generate the prime numbers with the sieve of Eratosthenes.
    """
    d = {}
    for i in itertools.count(2):
        if i in d:
            for j in d[i]:
                d[i + j] = d.get(i + j, []) + [j]
            del d[i]
        else:
            d[i * i] = [i]
            yield i

def eratosthenes_composite():
    """
    Generate the composite numbers with the sieve of Eratosthenes.
    """
    p1 = 3
    for p2 in eratosthenes():
        for n in range(p1 + 1, p2):
            yield n
        p1 = p2

def carmichael():
    for m in eratosthenes_composite():
        for a in range(2, m):
            if pow(a,m,m) != a:
                break
        else:
            yield m

def ackermann(m, n):
    """
    Ackermann number.
    """
    if m == 0:
        return n + 1
    elif n == 0:
        return ackermann(m - 1, 1)
    else:
        return ackermann(m - 1, ackermann(m, n - 1))

def fibonacci():
    """
    A generator for Fibonacci numbers, goes to next number in series on each call.
    This generator start at 1, 2, 3, 5, 8, 13, 21, 34, 55, 89, 144, 233, 377, 610, 987, 1597, 2584, 4181, 6765, 10946, ...
    See: http://oeis.org/A000045
    """
    a, b = 1, 2
    while True:
        yield a
        a, b = b, a + b

def syracuse(l=15):
    """
    Generate the sequence of Syracuse.
    """
    y = l
    while True:
        yield y
        q,r = divmod(y,2)
        if r == 0:
            y = q
        else:
            y = 3*y + 1

def log_gen():
    """
    Logarithmic generator.
    """
    y = 1
    while True:
        adder = max(1, math.pow(10, int(math.log10(y))))
        yield int(y)
        y = y + adder
        
if __name__ == "__main__":
    # Point of entry in execution mode.
    f = fibonacci()
    for x in range(13):
        print(next(f), end=' ') # 0 1 1 2 3 5 8 13 21 34 55 89 144



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Stéganô - Stéganô is a basic Python Steganography module.
# Copyright (C) 2010-2013  Cédric Bonhomme - http://cedricbonhomme.org/
#
# For more information : http://bitbucket.org/cedricbonhomme/stegano/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.1 $"
__date__ = "$Date: 2010/10/01 $"
__license__ = "GPLv3"

import base64
from functools import reduce

def a2bits(chars):
    """
    Converts a string to its bits representation as a string of 0's and 1's.

    >>> a2bits("Hello World!")
    '010010000110010101101100011011000110111100100000010101110110111101110010011011000110010000100001'
    """
    return bin(reduce(lambda x, y : (x<<8)+y, (ord(c) for c in chars), 1))[3:]

def a2bits_list(chars):
    """
    Convert a string to its bits representation as a list of 0's and 1's.

    >>>  a2bits_list("Hello World!")
    ['01001000',
    '01100101',
    '01101100',
    '01101100',
    '01101111',
    '00100000',
    '01010111',
    '01101111',
    '01110010',
    '01101100',
    '01100100',
    '00100001']
    >>> "".join(a2bits_list("Hello World!"))
    '010010000110010101101100011011000110111100100000010101110110111101110010011011000110010000100001'
    """
    return [bin(ord(x))[2:].rjust(8,"0") for x in chars]

def bs(s):
    """
    Converts an int to its bits representation as a string of 0's and 1's.
    """
    return str(s) if s<=1 else bs(s>>1) + str(s&1)

def setlsb(component, bit):
    """
    Set Least Significant Bit of a colour component.
    """
    return component & ~1 | int(bit)

def n_at_a_time(items, n, fillvalue):
    """
    Returns an iterator which groups n items at a time.
    Any final partial tuple will be padded with the fillvalue

    >>> list(n_at_a_time([1, 2, 3, 4, 5], 2, 'X'))
    [(1, 2), (3, 4), (5, 'X')]
    """
    it = iter(items)
    return its.izip_longest(*[it] * n, fillvalue=fillvalue)

def binary2base64(binary_file):
    """
    Convert a binary file (OGG, executable, etc.) to a
    printable string.
    """
    # Use mode = "rb" to read binary file
    fin = open(binary_file, "rb")
    binary_data = fin.read()
    fin.close()

    # Encode binary to base64 string (printable)
    return  base64.b64encode(binary_data)
    
    """fout = open(output_file, "w")
    fout.write(b64_data)
    fout.close"""

def base642binary(b64_fname):
    """
    Convert a printable file to a binary file.
    """
    # Read base64 string
    #fin = open(b64_fname, "r")
    #b64_str = fin.read()
    #fin.close()
    # Decode base64 string to original binary sound object
    return base64.b64decode(b64_fname)